<?php
/*
Plugin Name:  Business Booster CRM pro WPForms
Plugin URI:
Description:  Plugin pro napojení CRM systému Business Booster na WordPress plug-in pro kontaktní formuláře WPForms nebo WPForms Lite.
Version:      1.3.2.1
Author:       Expanzia s.r.o.
Author URI:
License:
License URI:
*/

function bbwp_plugin_activate(){
    if(bbwp_check_wpforms()){
        bbwp_link_wpforms(true);
        bbwp_load_translation();
    } else {
        bbwp_missing_dependency();
    }
}
register_activation_hook( __FILE__, 'bbwp_plugin_activate' );


function bbwp_plugin_deactivate(){
    if(bbwp_check_wpforms()){
        bbwp_unlink_wpforms(bbwp_wpforms_path());
    }
    $timestamp = wp_next_scheduled( 'bbwp_cron_hook' );
    wp_unschedule_event( $timestamp, 'bbwp_cron_hook' );
}
register_deactivation_hook( __FILE__, 'bbwp_plugin_deactivate' );

/**
 * Checks whether WPForms Lite|PRO is updated and if so, linked provider again
 * @param $upgrader_object
 * @param $options
 */
function bbwp_check_updates( $upgrader_object, $options ) {
    if ($options['action'] == 'update' && $options['type'] == 'plugin' ){
        foreach($options['plugins'] as $each_plugin){
            if (preg_match('/wpforms/i',$each_plugin)){
                bbwp_plugin_activate();
            }
        }
    }
}
add_action( 'upgrader_process_complete', 'bbwp_check_updates',10, 2);


function bbwp_check_dependencies(){
    if(!bbwp_check_wpforms()){
        bbwp_missing_dependency();
    }

    $providers = get_option('wpforms_providers');
    if (empty($providers['business-booster'])) {
        add_action('admin_notices', 'bbwp_wpforms_missing_provider');
    }
}
add_action('admin_init','bbwp_check_dependencies');

function bbwp_missing_dependency(){
    deactivate_plugins(plugin_basename( __FILE__ ), true);
    add_action('admin_notices','bbwp_wpforms_missing_notice');
}

function bbwp_connection_error(){
    deactivate_plugins(plugin_basename( __FILE__ ), true);
    add_action('admin_notices','bbwp_wpforms_wrong_permission');
}

add_filter('wpforms_load_providers', 'bbwp_load_provider');

function bbwp_load_provider($providers) {
    if(bbwp_check_wpforms()){
        bbwp_link_wpforms();

        if(file_exists(bbwp_provider_path())) {
            $providers[] = 'business-booster';
        }
    }
    return $providers;
}

function bbwp_load_translation(){
    $moFile = WP_LANG_DIR.'/plugins/wpforms-cs_CZ.mo';
    if(!file_exists($moFile)){
        if(is_writable(WP_LANG_DIR.'/plugins')) {
            copy(__DIR__ . '/languages/wpforms-cs_CZ.mo', $moFile);
        }
        else{
        add_action('admin_notices','bbwp_wpforms_wrong_permission');
            }
    }
}

function bbwp_check_wpforms(){
    include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
    return is_plugin_active('wpforms/wpforms.php') || is_plugin_active('wpforms-lite/wpforms.php');
}

function bbwp_function_enabled($functionName) {
    $disabled = explode(',', ini_get('disable_functions'));
    return !in_array($functionName, $disabled);
}

function bbwp_wpforms_path(){
     if(is_plugin_active('wpforms/wpforms.php')){
        return WP_PLUGIN_DIR.'/wpforms/';
     }
    if(is_plugin_active('wpforms-lite/wpforms.php')){
        return WP_PLUGIN_DIR.'/wpforms-lite/';
    }
}

function bbwp_provider_path(){
    return bbwp_wpforms_path().'includes/providers/class-business-booster.php';
}

function bbwp_link_wpforms($forceCopy = false){
    $source = WP_PLUGIN_DIR.'/business-booster-wpforms/class-business-booster.php';
    $dest = bbwp_provider_path();
    if(!file_exists($dest) || $forceCopy) {
        if(is_writeable(dirname($dest))) {
            copy($source,$dest);
        }
        else{
            bbwp_connection_error();
        }
    }
}

function bbwp_unlink_wpforms($path){
    $link = $path.'includes/providers/class-business-booster.php';
    if(file_exists($link)) {
        unlink($link);
    }
}


// Plugin updater
require 'vendor/plugin-update-checker/plugin-update-checker.php';
$myUpdateChecker = Puc_v4_Factory::buildUpdateChecker(
    'https://www.businessbooster.cz/plugins/?action=get_metadata&slug=business-booster-wpforms',
    __FILE__,
    'business-booster-wpforms'
);

//notices helpers
function bbwp_wpforms_missing_notice(){?>
    <div class="error notice">
        <p>
            Pro napojení na <strong>Business Booster CRM</strong> musíte mít nainstalován plugin
            <a href="https://wordpress.org/plugins/wpforms-lite/" target="_blank">WPForms Lite</a>
            nebo
            <a href="https://wpforms.com" target="_blank">WPForms Pro</a>.
        </p>
        <p>Nainstalujte a aktivujte nejprve jeden z uvedených pluginů a teprve poté můžete aktivovat plugin pro napojení na Business Booster CRM</p>
    </div>
<?php }

function bbwp_wpforms_wrong_permission(){?>
    <div class="error notice">
        <p>
            Do složky <pre><?= bbwp_wpforms_path();?></pre> nelze zapisovat. Nepodařilo se zkopírovat soubory.
        </p>
        <p>Pro používání pluginu povolte zápis do složky a plugin znovu aktivujte.</p>
    </div>
<?php }

function bbwp_wpforms_i18n_wrong_permission(){?>
    <div class="error notice">
        <p>
            Do složky <pre><?php WP_LANG_DIR;?></pre> nelze zapisovat. Nepodařilo se zkopírovat soubory s překlady.
        </p>
        <p>Pro používání překladů pluginu povolte zápis do složky a plugin znovu aktivujte.</p>
    </div>
<?php }

function bbwp_wpforms_missing_provider() {
    $class = 'notice notice-warning';
    $settingsPath = './admin.php?page=wpforms-settings&view=integrations';
    $message = <<<EOT
Pro správné ukládání odeslaných formulářů do Business Booster je třeba napojit stránky na WordPress.<br />
Napojení můžete provést v <a href="${settingsPath}">nastavení pluginu WP-Forms</a>. Též můžete využít podporu systému 
<a href="http://www.businessbooster.cz/podpora/kontaktni-formulare" target="_blank">Business Booster</a>.
EOT;
    printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($class), $message);

}

//CRON Task ===============================
/**
 * Checks if there are stored forms data - it means BusinessBooster was not available
 * We try to send it again
 */
function bbwp_process_transients() {
    global $wpdb;
    $results = $wpdb->get_results( "SELECT * FROM {$wpdb->prefix}options WHERE option_name like '_transient_bb2wpforms%'", OBJECT );
    if(!empty($results)) {
        foreach ($results as $result) {
            $name = str_replace("_transient_","",$result->option_name);
            $data = get_transient($name);
            bbwp_reprocess_entry($name,$data['api_endpoint'],$data['data']);
        }
    }
}

function bbwp_reprocess_entry($transient_name,$endpoint,$data){
    $response =
        wp_remote_post($endpoint, $data);
    $responseData = wp_remote_retrieve_body($response);
    //store as a transient if BB is not available
    if(!is_wp_error($response) && strpos($responseData,'Data was accepted') !== false){
        delete_transient($transient_name);
    } else {
        //notify plugin author that something went wrong
        $json = json_decode($data['body']['create_contact']);
        if(!$json){
            $json = json_decode($data);
        }
        bbwp_log_error("Error while storing data occurred in " . __FILE__ . ":" . __LINE__,
                       $json,
                       array(
                           'endpoint' => $endpoint,
                           'transient_name' => $transient_name,
                           'transient_data' => var_export($data, true),
                           'response' => $responseData
                       )
        );
    }
}

add_action('bbwp_cron_hook', 'bbwp_process_transients');
if (!wp_next_scheduled('bbwp_cron_hook')) {
    wp_schedule_event(time(), 'hourly', 'bbwp_cron_hook');
}


function bbwp_log_error($message,$json, $other = []){
    $dump = var_export($_SERVER, true);
    unset($json->contact);
    $domain = str_replace(array("http://", "https://", "www."), array("", "", ""), get_site_url());
    $headers = 'From: no-reply@' . $domain . "\r\n" .
               'X-Mailer: PHP/' . phpversion();

    $message .= "\n\n User: {$json->email}\n\nJSON DUMP ============\n\n" . json_encode($json) . "\n\nSERVER DUMP ==================\n\n $dump";

    foreach($other as $key => $value){
        $message .= "\n\n{$key} ==========\n${value}\n";
    }

    mail('frantisek@craftcoders.cz', 'Storing from in BB failed', $message, $headers);
}
