<?php

/**
 * Business Booster integration.
 *
 * @author     craftcoders
 * @license
 * @copyright
 */
class WPForms_Business_Booster extends WPForms_Provider {

    /**
     * Provider access token.
     *
     * @since 1.3.6
     * @var string
     */
    public $access_token;

    /**
     * BB login e-mail
     */
    public $email;

    /**
     * Provider API key.
     *
     * @since 1.3.6
     * @var string
     */
    public $api_key;

    public $api_endpoint = 'https://is.businessbooster-crm.com/external_forms';


    /**
     * Initialize.
     *
     * @since 1.3.6
     */
    public function init() {

        // Load form if found
        $form_id = isset($_REQUEST['form_id']) ? absint($_REQUEST['form_id']) : false;
        $this->form = wpforms()->form->get($form_id);
        $this->form_data = $this->form ? wpforms_decode($this->form->post_content) : false;

        $this->version = '1.4.5';
        $this->name = 'Business Booster CRM';
        $this->slug = 'business-booster';
        $this->priority = 14;
        $this->icon = plugins_url() . '/business-booster-wpforms/logo.png';

        //        $blogUrl = get_site_url(null, '','https');
        //        $blogUrl = str_replace('https://','',$blogUrl);
        //        $this->api_endpoint .= '?source='.$blogUrl;

        if (is_admin()) {
            // Admin notice requesting connecting
            //			add_action( 'admin_notices', array( $this, 'connect_request' ) );
            //			add_action( 'wp_ajax_wpforms_business_booster_dismiss', array( $this, 'connect_dismiss' ) );
            //			add_action( 'wpforms_admin_page', array( $this, 'learn_more_page' ) );

        }
    }

    /**
     * Process and submit entry to provider.
     * Provider request example: {"email":"pac@ebee.cz","token":"732e11b5c8aa8c93f49dff53caad2330","ip":"11.22.33.44","groups":[2],"newsletters":[10003866],"contact":{"email":"fandasson.pacson@gmail.com","name":"Františeksson","surname":"Pácsson"}}
     *
     * @param array $fields
     * @param array $entry
     * @param array $form_data
     * @param int $entry_id
     *
     * @return void
     * @since 1.3.6
     *
     */
    public function process_entry($fields, $entry, $form_data, $entry_id = 0) {

        // Only run if this form has a connections for this provider
        if (empty($form_data['providers'][$this->slug])) {
            return;
        }

        $providers = get_option('wpforms_providers');
        $results = array();

        // Fire for each connection ------------------------------------------//


        foreach ($form_data['providers'][$this->slug] as $connection) {

            // Before proceeding make sure required fields are configured
            if (empty($connection['fields']['email'])) {
                continue;
            }
            // Setup basic data
            $list_id = $connection['list_id'];
            $source_id = $connection['source_id'];
            $newsletter_id = $connection['newsletter_id'];
            $account_id = $connection['account_id'];
            $data = array(
                'groups' => "[$list_id]",
                'source' => "$source_id",
                'newsletters' => "[$newsletter_id]"
            );


            // Check for conditionals
            $pass = $this->process_conditionals($fields, $entry, $form_data, $connection);
            if (!$pass) {
                wpforms_log(__('Business Booster Subscription stopped by conditional logic', 'wpforms'), $fields, array(
                    'type' => array('provider', 'conditional_logic'),
                    'parent' => $entry_id,
                    'form_id' => $form_data['id'],
                )
                );
                continue;
            }


            // Setup Merge Vars ---------------------------------------------//
            $merge_vars = array();

            foreach ($connection['fields'] as $name => $merge_var) {


                // Check if merge var is mapped
                if (empty($merge_var)) {
                    continue;
                }

                $merge_var = explode('.', $merge_var);
                $id = $merge_var[0];
                $key = !empty($merge_var[1]) ? $merge_var[1] : 'value';
                $type = !empty($merge_var[2]) ? $merge_var[2] : 'text';

                // Check if mapped form field has a value
                if (empty($fields[$id][$key])) {
                    continue;
                }

                $value = $fields[$id][$key];


                $merge_vars[$name] = $value;
            }

            // Process in API ------------------------------------------------//

            // Combine merge vars into data before sending
            $data = array_merge($data, array('contact' => $merge_vars));

            $args = $this->prepareConnection($account_id, 'create_contact', $data);
            $response =
                wp_remote_post($this->api_endpoint, $args);
            $responseData = wp_remote_retrieve_body($response);

            //store as a transient if BB is not available
            if (is_wp_error($response) || strpos($responseData, 'Data was accepted') === false) {
                $transient_name = 'bb2wpforms_' . md5(implode("|", $args));
                $data = array(
                    'api_endpoint' => $this->api_endpoint,
                    'data' => $args
                );

                set_transient($transient_name, $data, WEEK_IN_SECONDS);
                //notify author that something went wrong
                $json = json_decode($data['data']['body']['create_contact']);
                bbwp_log_error("Error while storing data occurred in " . __FILE__ . ":" . __LINE__, $json,
                array('endpoint' => $this->api_endpoint,
                        'response' => $response,
                        'responseData' => $responseData,
                        'transient' => $transient_name
                )
                );
            }
        }
    }

    // ************************************************************************//
    //
    // API methods - these methods interact directly with the provider API.
    //
    // ************************************************************************//

    /**
     * Authenticate with the API.
     *
     * @param array $data
     * @param string $form_id
     *
     * @return mixed id or error object
     * @since 1.3.6
     *
     */
    public function api_auth($data = array(), $form_id = '') {

        $id = uniqid();
        $providers = get_option('wpforms_providers', array());

        $email = sanitize_text_field($data['email']);
        $providers[$this->slug][$id] = array(
            'api_key' => sanitize_text_field($data['api_key']),
            'email' => $email,
            'label' => 'Business Booster CRM (' . $email . ')',
            'date' => time(),
        );
        update_option('wpforms_providers', $providers);

        return $id;
    }

    /**
     * Establish connection object to API.
     *
     * @param string $account_id
     *
     * @return mixed array or error object.
     * @since 1.3.6
     *
     */
    public function api_connect($account_id) {

        if (!empty($this->api[$account_id])) {
            return $this->api[$account_id];
        } else {
            $providers = get_option('wpforms_providers');
            if (!empty($providers[$this->slug][$account_id])) {
                $this->api[$account_id] = true;
                $this->api_key = $providers[$this->slug][$account_id]['api_key'];
                $this->email = $providers[$this->slug][$account_id]['email'];
                $this->access_token = $this->createToken();
            } else {
                return $this->error('API error');
            }
        }
    }

    private function createToken() {
        return md5($this->api_key . '*' . $this->email);
    }

    /**
     * Retrieve provider account lists.
     *
     * @param string $connection_id
     * @param string $account_id
     *
     * @return mixed array or error object
     * @since 1.3.6
     *
     */
    public function api_lists($connection_id = '', $account_id = '') {
        return $this->api_load_list($account_id, 'get_groups');
    }

    public function api_sources($account_id = '') {
        return $this->api_load_list($account_id, 'get_sources');
    }

    public function api_newsletters($account_id = '') {
        return $this->api_load_list($account_id, 'get_newsletters');
    }

    private function api_load_list($account_id, $list) {
        $args = $this->prepareConnection($account_id, $list, array());

        $request = wp_remote_post($this->api_endpoint, $args);
        $res = json_decode(wp_remote_retrieve_body($request), true);
        $lists = array();
        $lists[] = array(); //first empty

        if (!empty($res)) {
            foreach ($res as $r) {
                $lists[] = array(
                    'id' => $r[0],
                    'name' => $r[1],
                    'status' => 'ACTIVE'
                );
            }
        }
        return $lists;
    }

    /**
     * Retrieve provider account list fields.
     *
     * @param string $connection_id
     * @param string $account_id
     * @param string $list_id
     *
     * @return mixed array or error object
     * @since 1.3.6
     *
     */
    public function api_fields($connection_id = '', $account_id = '', $list_id = '') {

        $provider_fields = array(
            array(
                'name' => 'E-mail',
                'field_type' => 'email',
                'tag' => 'email',
            ),
            array(
                'name' => 'E-mail 2',
                'field_type' => 'email',
                'tag' => 'email2',
            ),
            array(
                'name' => 'Titul (před jménem)',
                'field_type' => 'text',
                'tag' => 'degree',
            ),
            array(
                'name' => 'Titul (za jménem)',
                'field_type' => 'text',
                'tag' => 'degree_suffix',
            ),
            array(
                'name' => 'Jméno',
                'field_type' => 'first',
                'tag' => 'name',
            ),
            array(
                'name' => 'Příjmení',
                'field_type' => 'last',
                'tag' => 'surname',
            ),
            array(
                'name' => 'Telefon',
                'field_type' => 'text',
                'tag' => 'phone',
            ),
            array(
                'name' => 'Mobil',
                'field_type' => 'text',
                'tag' => 'gsm',
            ),
            array(
                'name' => 'WWW stránky',
                'field_type' => 'text',
                'tag' => 'url',
            ),
            array(
                'name' => 'Facebook profil',
                'field_type' => 'text',
                'tag' => 'facebook_url',
            ),
            array(
                'name' => 'Google+ profil',
                'field_type' => 'text',
                'tag' => 'googleplus_url',
            ),
            array(
                'name' => 'LinkedIn profil',
                'field_type' => 'text',
                'tag' => 'linkedin_url',
            ),
            array(
                'name' => 'Společnost',
                'field_type' => 'text',
                'tag' => 'company',
            ),
            array(
                'name' => 'Pozice',
                'field_type' => 'text',
                'tag' => 'function',
            ),
            array(
                'name' => 'IČO osobní',
                'field_type' => 'text',
                'tag' => 'ico_physical',
            ),
            array(
                'name' => 'IČO firemní',
                'field_type' => 'text',
                'tag' => 'ico_business',
            ),
            array(
                'name' => 'Poznámka',
                'field_type' => 'text',
                'tag' => 'notice',
            )
        );

        return $provider_fields;
    }

    function prepareConnection($account_id, $method, $data) {
        $this->api_connect($account_id);

        if (empty($this->access_token)) {
            //todo better error handling
            return array();
        }

        $accessData = array('email' => $this->email, 'token' => $this->access_token, 'ip' => $_SERVER['REMOTE_ADDR']);
        $args = array(
            'body' => array($method => json_encode(array_merge($accessData, $data))),
            'sslverify' => false

        );

        return $args;
    }


    // ************************************************************************//
    //
    // Output methods - these methods generally return HTML for the builder.
    //
    // ************************************************************************//


    /**
     * Provider account list groups HTML.
     *
     * @param string $connection_id
     * @param array $connection
     *
     * @return string
     * @since 1.3.6
     *
     */
    public function output_groups($connection_id = '', $connection = array()) {
        // No groups or segments for this provider
        return;
    }

    /**
     * Output content after the main builder output.
     *
     * @since 1.3.6
     */
    public function builder_output_after() {

        // Only display if Constant Contact account has not been setup
        $providers = get_option('wpforms_providers', array());

        if (!empty($providers[$this->slug])) {
            return;
        }
        ?>
        <div class="wpforms-alert wpforms-alert-info">
            <p><?php _e('Get the most out of <strong>WPForms</strong> &mdash; use it with an active Constant Contact account.',
                        'wpforms'
                ); ?></p>
            <p>
                <a href="<?php echo $this->sign_up; ?>" style="margin-right: 10px;" class="button-primary"
                   target="_blank"
                   rel="noopener noreferrer"><?php _e('Try Constant Contact for Free', 'wpforms'); ?></a>
                <?php printf(__('Learn More about the <a href="%s" target="_blank" rel="noopener noreferrer">power of email marketing</a>',
                                'wpforms'
                             ), admin_url('admin.php?page=wpforms-page&wpforms-page=constant-contact')
                ); ?>
            </p>
        </div>
        <?php
    }

    /**
     * Provider account lists HTML.
     *
     * @param string $connection_id
     * @param array $connection
     *
     * @return WP_Error|string
     * @since 1.0.0
     *
     */
    public function output_lists($connection_id = '', $connection = array()) {

        if (empty($connection_id) || empty($connection['account_id'])) {
            return '';
        }

        $lists = $this->api_lists($connection_id, $connection['account_id']);
        $sources = $this->api_sources($connection['account_id']);
        $newsletters = $this->api_newsletters($connection['account_id']);

        if (is_wp_error($lists)) {
            return $lists;
        }

        $output = '<div class="wpforms-provider-lists wpforms-connection-block">';

        $output .= sprintf('<h4>%s</h4>', __('Zdroj kontaktu', 'wpforms'));
        $output .= sprintf('<select name="providers[%s][%s][source_id]">', $this->slug, $connection_id);
        $output .= $this->output_list_part($connection, $sources, 'source_id');
        $output .= '</select>';

        $output .= sprintf('<h4>%s</h4>', __('Select List', 'wpforms'));
        $output .= sprintf('<select name="providers[%s][%s][list_id]">', $this->slug, $connection_id);
        $output .= $this->output_list_part($connection, $lists, 'list_id');
        $output .= '</select>';

        $output .= sprintf('<h4>%s</h4>', __('Newsletter', 'wpforms'));
        $output .= sprintf('<select name="providers[%s][%s][newsletter_id]">', $this->slug, $connection_id);
        $output .= $this->output_list_part($connection, $newsletters, 'newsletter_id');
        $output .= '</select>';

        $output .= '</div>';

        return $output;
    }

    private function output_list_part($connection, $lists, $id) {
        $output = "";
        $selected = !empty($connection[$id]) ? $connection[$id] : '';
        if (!empty($lists)) {
            foreach ($lists as $list) {
                $output .= sprintf(
                    '<option value="%s" %s>%s</option>',
                    esc_attr($list['id']),
                    selected($selected, $list['id'], false),
                    esc_attr($list['name'])
                );
            }
        }
        return $output;
    }

    /**
     * Provider account list fields HTML.
     *
     * @param string $connection_id
     * @param array $connection
     * @param mixed $form
     *
     * @return WP_Error|string
     * @since 1.0.0
     *
     */

    public function output_fields($connection_id = '', $connection = array(), $form = '') {

        if (empty($connection_id) || empty($connection['account_id']) || empty($form)) {
            return '';
        }

        if (empty($connection['fields'])) {
            $provider = $this->form_data['providers'][$this->slug][$connection_id];
            $connection['fields'] = $provider['fields'];
        }
        $provider_fields = $this->api_fields($connection_id, $connection['account_id'], $connection['list_id']);
        $form_fields = $this->get_form_fields($form);
        if (is_wp_error($provider_fields)) {
            return $provider_fields;
        }

        $output = '<div class="wpforms-provider-fields wpforms-connection-block">';

        $output .= sprintf('<h4>%s</h4>', __('List Fields', 'wpforms'));

        // Table with all the fields
        $output .= '<table>';

        $output .= sprintf('<thead><tr><th>%s</th><th>%s</th></thead>', __('List Fields', 'wpforms'),
                           __('Available Form Fields', 'wpforms')
        );

        $output .= '<tbody>';

        foreach ($provider_fields as $provider_field) :

            $output .= '<tr>';

            $output .= '<td>';

            $output .= esc_html($provider_field['name']);
            if (
                !empty($provider_field['req']) &&
                $provider_field['req'] == '1'
            ) {
                $output .= '<span class="required">*</span>';
            }

            $output .= '<td>';

            $output .= sprintf('<select name="providers[%s][%s][fields][%s]">', $this->slug, $connection_id,
                               esc_attr($provider_field['tag'])
            );

            $output .= '<option value=""></option>';

            $options = $this->get_form_field_select($form_fields, $provider_field['field_type']);

            foreach ($options as $option) {
                $value = sprintf('%d.%s.%s', $option['id'], $option['key'], $option['provider_type']);
                $selected =
                    !empty($connection['fields'][$provider_field['tag']]) ? selected($connection['fields'][$provider_field['tag']],
                                                                                     $value, false
                    ) : '';
                $output .= sprintf('<option value="%s" %s>%s</option>', esc_attr($value), $selected,
                                   esc_html($option['label'])
                );
            }

            $output .= '</select>';

            $output .= '</td>';

            $output .= '</tr>';

        endforeach;

        $output .= '</tbody>';

        $output .= '</table>';

        $output .= '</div>';

        return $output;
    }

    // ************************************************************************//
    //
    // Integrations tab methods - these methods relate to the settings page.
    //
    // ************************************************************************//

    /**
     * Form fields to add a new provider account.
     * Called from WPForms_Provide.integrations_tab_options
     * @since 1.3.6
     */
    public function integrations_tab_new_form() {

        $output = '<p>Pro napojení formulářů s vaším účtem Business Booster zadejte níže e-mail, s jakým se do systému Business Booster přihlašujete a kód pro externí formuláře. Kód naleznete ve vašem účtu Business Booster v záložce "Nastavení → Kontaktní formuláře.</p>
                    <p>Více informací naleznete na <a href="http://www.businessbooster.cz/podpora/kontaktni-formulare" target="_blank">www.businessbooster.cz/podpora/kontaktni-formulare</a>.</p>';

        $output .= '<p class="wpforms-alert wpforms-alert-warning">';
        $output .= '</p>';

        $output .= sprintf('<input type="text" name="email" placeholder="%s" class="wpforms-required">',
                           'Přihlašovací e-mail do Business Booster CRM'
        );

        $output .= sprintf('<input type="text" name="api_key"  placeholder="%s" class="wpforms-required">',
                           'Kód pro externí formuláře'
        );

        echo $output;
    }

    // ************************************************************************//
    //
    // Other functionality.
    //
    // ************************************************************************//

}

new WPForms_Business_Booster;
